# -*- coding: utf-8 -*-
import abc
from typing import Any, Dict
from urllib.parse import urlencode

from yunpian.abc import Requester, Response


class Client(abc.ABC):
    CONTENT_TYPE = 'application/x-www-form-urlencoded'

    def __init__(self, requester: Requester, api_key: str):
        self._api_key = api_key
        self.requester = requester

    async def _make_request(self, method: str, url: str, url_params: Dict,
                            data: Any) -> Response:
        filled_url = format_url(url, url_params)
        request_headers = create_headers(self._api_key)

        if isinstance(data, dict):
            request_headers['content-type'] = self.CONTENT_TYPE
            body = urlencode(data)
        else:
            body = data
        response = await self.requester.request(
            method, filled_url, request_headers, body
        )
        return response

    async def get(self, url: str, url_params: Dict) -> Response:
        return await self._make_request('GET', url, url_params, b'')

    async def post(self, url: str, url_params: Dict, *, data: Any) -> Response:
        return await self._make_request('POST', url, url_params, data)

    async def put(self, url: str, url_params: Dict, *, data: Any) -> Response:
        return await self._make_request('PUT', url, url_params, data)

    async def patch(self, url: str, url_params: Dict, *, data: Any
                    ) -> Response:
        return await self._make_request('PATCH', url, url_params, data)

    async def delete(self, url: str, url_params: Dict):
        return await self._make_request('DELETE', url, url_params, b'')


def format_url(url, url_params):
    return url + '?' + urlencode(url_params)


def create_headers(api_key: str) -> Dict[str, str]:
    headers = {
        'user-agent': 'python-yunpian/0.1.0',
    }
    return headers



# -*- coding: utf-8 -*-
from enum import Enum

from yunpian.core import Client
from yunpian.utils import clean_none


class URL(Enum):
    SINGLE_SEND = 'https://sms.yunpian.com/v2/sms/single_send.json'


class SMSClient(Client):
    """封装短信相关 API"""

    async def single_send(self, mobile: str, text: str, *,
                          extend: str = None, uid: str = None,
                          callback_url: str = None,
                          register: bool = None,
                          mobile_stat: bool = None,
                          api_url: str = URL.SINGLE_SEND.value):
        """单条发送接口

        https://www.yunpian.com/api2.0/api-domestic/single_send.html

        :param mobile:
        :param text:
        :param extend:
        :param uid:
        :param callback_url:
        :param register:
        :param mobile_stat:
        :param api_url:
        :return:
        """
        data = {
            'apikey': self._api_key,
            'mobile': mobile,
            'text': text,
            'extend': extend,
            'uid': uid,
            'callback_url': callback_url,
            'register': register,
            'mobile_stat': mobile_stat,
        }
        return await self.post(api_url, {}, data=clean_none(data))



# -*- coding: utf-8 -*-
import asyncio
from typing import Any, Dict

import aiohttp

from yunpian.abc import Requester, Response


class Aiohttp(Requester):
    def __init__(self, session: aiohttp.ClientSession, *args: Any,
                 **kwargs: Any) -> None:
        self._session = session
        super().__init__(*args, **kwargs)

    async def request(self, method: str, url: str, headers: Dict,
                      body: bytes = b'') -> Response:
        async with self._session.request(method, url, headers=headers,
                                         data=body) as response:
            return Response(
                status=response.status,
                headers=response.headers,
                body=await response.read(),
            )

    async def sleep(self, seconds: float) -> None:
        await asyncio.sleep(seconds)



# -*- coding: utf-8 -*-
import abc
import json
from typing import Dict

import attr


@attr.s
class Response:
    status: str = attr.ib()
    headers: Dict = attr.ib()
    body: bytes = attr.ib()

    def json(self):
        return json.loads(self.body)


class Requester(abc.ABC):

    def __init__(self, *args, **kwargs):
        pass

    @abc.abstractmethod
    async def request(self, method: str, url: str, headers: Dict,
                      body: bytes = b'') -> Response:
        pass

    @abc.abstractmethod
    async def sleep(self, seconds: float):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from codecs import open
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import os
import re


current_dir = os.path.dirname(os.path.realpath(__file__))
with open('README.rst', encoding='utf-8') as readme_file:
    readme = readme_file.read()


def get_meta():
    meta_re = re.compile(r"(?P<name>__\w+__) = '(?P<value>[^']+)'")
    meta_d = {}
    with open(os.path.join(current_dir, 'yunpian/__init__.py'),
              encoding='utf8') as fp:
        for match in meta_re.finditer(fp.read()):
            meta_d[match.group('name')] = match.group('value')
    return meta_d


requirements = [
    # TODO: put package requirements here
]

extras_require = {
    # ':python_version<"2.7"': ['argparse'],
    # ':python_version<"3.4"': ['enum34'],
}

meta_d = get_meta()
setup(
    name='yunpian',
    version=meta_d['__version__'],
    description='An async yunpian API library for Python',
    long_description=readme,
    author='mozillazg',
    author_email='mozillazg101@gmail.com',
    url='https://github.com/mozillazg/yunpian',
    packages=[
        'yunpian',
        'yunpian.contrib',
    ],
    package_dir={'yunpian':
                 'yunpian'},
    include_package_data=True,
    install_requires=requirements,
    extras_require=extras_require,
    license='MIT',
    zip_safe=False,
    # entry_points={
    #     'console_scripts': [
    #         'yunpian = yunpian.__main__:main',
    #     ],
    # },
    keywords='yunpian',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
)
